#pragma once
#include <unordered_map>
#include <string>
#include <algorithm>

//Copyright (C) cocosimulator.org 2014
//generated by AmsterCHEM's COM CAPE-OPEN Wizard

//key - basic type. This type stores the value, unless its derived class is StringMapKeyPtr
class StringMapKey {
	public:
	wstring keyVal; //note that this is not used for StringMapKeyPtr
	virtual operator const wchar_t *() const {return keyVal.c_str();}
	StringMapKey() {}
	StringMapKey(const StringMapKey &other) {keyVal=other;}
	const StringMapKey &operator=(const StringMapKey &other) {keyVal=other;return *this;}
	StringMapKey(const wchar_t *key) {keyVal=key;}
	StringMapKey(const wstring &key) {keyVal=key;}
};

//key if we have a wstring value

class StringMapKeyString : public StringMapKey {
	public:
	StringMapKeyString(const StringMapKey &other) : StringMapKey(other) {};
	StringMapKeyString(const wchar_t *key) : StringMapKey(key) {};
	StringMapKeyString(const wstring &key) : StringMapKey(key) {};
	wstring::iterator begin() {return keyVal.begin();}
	wstring::iterator end() {return keyVal.end();}
};

//key for look-ups, holding a temporary const wchar_t *

class StringMapKeyPtr : public StringMapKey {
	const wchar_t *keyVal;
	public:
	StringMapKeyPtr(const wchar_t *key) {keyVal=key;}
	StringMapKeyPtr(const wstring &key) {keyVal=key.c_str();}
	operator const wchar_t *() const override {return keyVal;}
};

//equality test

struct StringMapKeyEquals {
	bool operator()(StringMapKey const& x, StringMapKey const& y) const {
		//the keys are stored as lower case
		return wcscmp(x,y)==0;
	}
};

//hasher

struct StringMapKeyHasher  {
	size_t operator()(StringMapKey const& x) const {
	    const wchar_t *ptr=x;
		size_t _Val = 2166136261U;
		while (*ptr) {
		   _Val = 16777619U * _Val ^ (size_t)*ptr;
		   ptr++;
		}
		return _Val;
	}
};

//the map

template <class ValueType> class StringMap {
private:
	using MAP = unordered_map<StringMapKey,ValueType,StringMapKeyHasher,StringMapKeyEquals>;	
	MAP map;

public:

	//note: use Lookup, not [], for efficient finding. May save a heap allocation of the key

	ValueType& operator[](const wchar_t * key) {
		//presume this is for setting... 
		StringMapKeyString k(key);
		transform(k.begin(),k.end(),k.begin(),::tolower);
		return map[k];
	}

	ValueType& operator[](const wstring & key) {
		//presume this is for setting... 
		StringMapKeyString k(key);
		transform(k.begin(),k.end(),k.begin(),::tolower);
		return map[k];
	}

	bool Lookup(const wchar_t *key,ValueType &value) const {
		MakeLowerCase lower(key);
		StringMapKeyPtr k(lower);
		typename MAP::const_iterator it;
		it=map.find(k);
		if (it==map.end()) {
			return false;
		}
		value=it->second;
		return true;
	}

	bool Lookup(const wstring &key,ValueType &value) const {
		MakeLowerCase lower(key);
		StringMapKeyPtr k(lower);
		typename MAP::const_iterator it;
		it=map.find(k);
		if (it==map.end()) {
			return false;
		}
		value=it->second;
		return true;
	}

	bool LookupRef(const wchar_t *key,const ValueType *&valueRef) const {
		MakeLowerCase lower(key);
		StringMapKeyPtr k(lower);
		typename MAP::const_iterator it;
		it=map.find(k);
		if (it==map.end()) {
			return false;
		}
		valueRef=&it->second;
		return true;
	}

	bool LookupRef(const wstring &key,const ValueType *&valueRef) const  {
		MakeLowerCase lower(key);
		StringMapKeyPtr k(lower);
		typename MAP::const_iterator it;
		it=map.find(k);
		if (it==map.end()) {
			return false;
		}
		valueRef=&it->second;
		return true;
	}

	int Count() {
		return (int)map.size();
	}

	bool ReverseLookup(ValueType &value,wstring &key) {
		//only ok for ValueType for which == operates as expected
		// obviously not an efficient lookup
		typename MAP::iterator it;
		it=map.begin();
		while (it!=map.end()) {
			if (it->second==value) {
				key=it->first;
				return true;
			}
			it++;
		}
		return false;
	}

	void Clear() {
		map.clear();
	}

	void DeleteAllAndClear() {
		//only works for classes
		typename MAP::iterator it;
		it=map.begin();
		while (it!=map.end()) {
			delete it->second;
			it++;
		}
		map.clear();
	}

	void Remove(const wchar_t *key) {
		MakeLowerCase lower(key);
		StringMapKeyPtr k(lower);
		typename MAP::iterator it;
		it=map.find(k);
		if (it!=map.end()) {
			map.erase(it);
		}
	}

	void AllItems(vector<const wchar_t *> &keys,vector<const ValueType> &values) {
		keys.clear();
		keys.reserve(map.size());
		values.clear();
		values.reserve(map.size());
		typename MAP::iterator it;
		it=map.begin();
		while (it!=map.end()) {
			keys.push_back(it->first());
			values.push_back(it->second);
			it++;
		}
	}

	template <class F> void Operate(F f) {
		/*F should be of form: 
				bool f(const wstring &key,const ValueType &value);
			and should return false to stop iterating
		*/
		typename MAP::iterator it;
		it=map.begin();
		while (it!=map.end()) {
			if (!f(it->first.keyVal,it->second)) break;
			it++;
		}
		
	}

};
