#pragma once
#include <cmath>
typedef vector<wstring> stringarray;

//Copyright (C) cocosimulator.org 2014
//generated by AmsterCHEM's COM CAPE-OPEN Wizard

//CVariant class: wrapper class for VARIANT values

class CVariant {

private:

	VARIANT value;
	bool mustDelete;
	//for array variables:
	LONG lBound,count;
	//keep track of element type during debugging
#ifdef _DEBUG
	VARTYPE elementType;
#endif

public:

	//default constructor (result will be destroyed)
	CVariant() {
		VariantInit(&value);
		mustDelete=true;
		count=0;
#ifdef _DEBUG
		elementType=VT_EMPTY;
#endif
	}

	//when an existing VARIANT needs to be analyzed, use this constructor
	// you must indicate whether or not the result must be destroyed
	CVariant(VARIANT value,bool DestroyWhenDone) {
		mustDelete=DestroyWhenDone;
		this->value=value;
		count=0;
#ifdef _DEBUG
		elementType=VT_EMPTY; //not checked
#endif
	}

	~CVariant() {
		//destructor
		if (mustDelete) {
			VariantClear(&value);
		}
	}

	//for the assignment from another CVariant, we presume this
	// is the result of a return value of a function, such as
	// CVariant::GetVariantAt; we transfer the requirement to 
	// destroy the content to the target

	const CVariant & operator = (CVariant &other) {
		if (mustDelete) {
			VariantClear(&value);
		}
		value=other.value;
		mustDelete=other.mustDelete;
		other.mustDelete=false;
		return *this;
	}

	//same for the copy constructor:

	CVariant(CVariant &other) {
		if (mustDelete) {
			VariantClear(&value);
		}
		value=other.value;
		mustDelete=other.mustDelete;
		other.mustDelete=false;
	}

	//when a VARIANT return value is obtained, use this function.
	// the result will then be destroyed, e.g. pass to functions
	//  that take a [out, retval] argument
	VARIANT *OutputArgument() {
		//used to pass variant to other routine as output argument
		if (mustDelete) {
			VariantClear(&value);
		} else {
			value.vt=VT_EMPTY;
		}
		mustDelete=true;
		return &value;
	}

	bool IsEmpty() {
		return (value.vt==VT_EMPTY);
	}

	//create a variant array; arrays that are created always index from 0
	void MakeArray(int count,VARTYPE vt) {
		SAFEARRAYBOUND ba;
		//clean up old content
		if (mustDelete)
			if (value.vt!=VT_EMPTY) {
				VariantClear(&value);
			}
		//set new content
		if (count==0) {
			value.vt=VT_EMPTY;
		} else {
			mustDelete=true;
			ba.cElements=count;
			ba.lLbound=0;
			value.parray=SafeArrayCreate(vt,1,&ba);
			value.vt=VT_ARRAY|vt;
			//keep track of content
			lBound=0;
			this->count=count;
		}
#ifdef _DEBUG
		elementType=vt;
#endif
	}

	//when a VARIANT return value is requested, use this function, as the return value
	// of a [out, retval] argument
	// the result will then not be destroyed
	__inline VARIANT ReturnValue() {
		mustDelete=false;
		return value;
	}

	//number of elements in an array:
	__inline int GetCount() {
		return count;
	}
	
	//attach and detach

	void Attach(VARIANT value) {
		//attach
		//	note that count is only valid after CheckArray
		Set(value,true);
	}

	void Detach() {
		//detach without destroying
		mustDelete=false;
		value.vt=VT_EMPTY;
	}

	//functions to reset the content to something else (re-use a CVariant)

	void Set(VARIANT value,bool DestroyWhenDone) {
		//	note that count is only valid after CheckArray
		//clean up old content
		if (mustDelete)
			if (this->value.vt!=VT_EMPTY) {
				VariantClear(&this->value);
			}
		//set to existing VARIANT
		mustDelete=DestroyWhenDone;
		this->value=value;
		count=0;
#ifdef _DEBUG
		elementType=VT_EMPTY; //not checked
#endif
	}

	void Clear() {
		//clean up old content
		if (mustDelete) {
			VariantClear(&value);
		}
		value.vt=VT_EMPTY;
		count=0;
	}

	//accessing and setting array elements

	__inline void SetBoolAt(LONG index,VARIANT_BOOL b) {
		//make sure to have called MakeArray before this
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		ASSERT(lBound==0,"invalid CVariant lower bound"); //saves an operation; all MakeArray values have lBound=0
		ASSERT(elementType==VT_BOOL,"invalid CVariant type");
		SafeArrayPutElement(value.parray,&index,&b);
	}

	__inline VARIANT_BOOL GetBoolAt(LONG index) {
		//make sure CheckArray was called before this
		VARIANT_BOOL b;
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		ASSERT(elementType==VT_BOOL,"invalid CVariant type");
		index+=lBound;
		SafeArrayGetElement(value.parray,&index,&b);
		return b;
	}

	__inline void SetDoubleAt(LONG index,double d) {
		//make sure to have called MakeArray before this
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		ASSERT(lBound==0,"invalid CVariant lower bound"); //saves an operation; all MakeArray values have lBound=0
		ASSERT(elementType==VT_R8,"invalid CVariant type");
		SafeArrayPutElement(value.parray,&index,&d);
	}

	__inline double GetDoubleAt(LONG index) {
		//make sure CheckArray was called before this
		double d;
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		ASSERT(elementType==VT_R8,"invalid CVariant type");
		index+=lBound;
		SafeArrayGetElement(value.parray,&index,&d);
		return d;
	}

	__inline void SetStringAt(LONG index,BSTR b) {
		//make sure to have called MakeArray before this
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		ASSERT(lBound==0,"invalid CVariant lower bound"); //saves an operation; all MakeArray values have lBound=0
		ASSERT(elementType==VT_BSTR,"invalid CVariant type");
		SafeArrayPutElement(value.parray,&index,b);
	}

	__inline CBStr GetStringAt(LONG index) {
		//make sure CheckArray was called before this
		CBStr b;
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		ASSERT(elementType==VT_BSTR,"invalid CVariant type");
		index+=lBound;
		SafeArrayGetElement(value.parray,&index,&b);
		return b;
	}

	__inline void SetVariantAt(LONG index,VARIANT v) {
		//make sure to have called MakeArray before this
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		ASSERT(lBound==0,"invalid CVariant lower bound"); //saves an operation; all MakeArray values have lBound=0
		ASSERT(elementType==VT_VARIANT,"invalid CVariant type");
		SafeArrayPutElement(value.parray,&index,&v);
	}

	__inline CVariant GetVariantAt(LONG index) const {
		CVariant v;
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		ASSERT(elementType==VT_VARIANT,"invalid CVariant type");
		index+=lBound;
		SafeArrayGetElement(value.parray,&index,v.OutputArgument());
		return v;
	}

	__inline void SetLongAt(LONG index,LONG i) {
		//make sure to have called MakeArray before this
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		ASSERT(lBound==0,"invalid CVariant lower bound"); //saves an operation; all MakeArray values have lBound=0
		ASSERT(elementType==VT_I4,"invalid CVariant type");
		SafeArrayPutElement(value.parray,&index,&i);
	}

	__inline LONG GetLongAt(LONG index) {
		//make sure CheckArray was called before this
		LONG i;
		ASSERT((count>0)&&(index<count),"CVariant index out of range");
		index+=lBound;
		ASSERT(elementType==VT_I4,"invalid CVariant type");
		SafeArrayGetElement(value.parray,&index,&i);
		return i;
	}

	__inline VARIANT Copy() const {
		//return a copy of the current value
		// (destroying is responsibility of caller)
		VARIANT v;
		v.vt=VT_EMPTY;
		VariantCopy(&v,(VARIANT *)&value); //cast required for MinGW; it has the typedef of VariantCopy wrong (second arg should be const)
		return v;
	}

	__inline VARIANT Value() {
		//this will not destroy the value, use only for values we
		// own, such as setting values with SetSinglePhaseProp
		return value;
	}

	//utility functions

	bool CheckArray(VARTYPE vt,wstring &error) {
		//sets the low bound and count if check is
#ifdef _DEBUG
		elementType=vt;
#endif
		// performed ok
		bool res=false;
		//check type
		if (value.vt==VT_EMPTY) {
			//this is ok for an empty array
			res=true;
			lBound=count=0;
		} else if (value.vt!=(vt|VT_ARRAY)) {
			//not the type we expected
			error=L"Expected an array of type ";
			switch (vt) {
			case VT_R8:
				error+=L"double precision";
				break;
			case VT_VARIANT:
				error+=L"VARIANT";
				break;
			case VT_BSTR:
				error+=L"string";
				break;
			case VT_I4:
				error+=L"long";
				break;
			case VT_I2:
				error+=L"short";
				break;
			default:
				error+=L"<unknown>";
				ASSERT(0,"CVariant does not implement this type");
				break;
			}
		} else {
			//type is ok, check dimension
			if (SafeArrayGetDim(value.parray)!=1) {
				error=L"unexpected dimension; 1-dimensional array expected";
			} else {
				//get lower and upper bound:
				if (FAILED(SafeArrayGetLBound(value.parray,1,&lBound))) {
					error=L"failed to get lower bound";
				} else if (FAILED(SafeArrayGetUBound(value.parray,1,&count))) {
					error=L"failed to get upper bound";
				} else {
					//all ok
					count-=lBound-1;
					res=true;
				}
			}
		}
		return res;
	}

	//Set 

	void Set(const OLECHAR *str) {
		//set to BSTR
		if (mustDelete)
			if (this->value.vt!=VT_EMPTY) {
				VariantClear(&this->value);
			}
		mustDelete=true;
		value.vt=VT_BSTR;
		value.bstrVal=(str)?SysAllocString(str):NULL;		
	}

	void Set(const double d) {
		//set to real
		if (mustDelete)
			if (this->value.vt!=VT_EMPTY) {
				VariantClear(&this->value);
			}
		mustDelete=true;
		value.vt=VT_R8;
		value.dblVal=d;		
	}

	void Set(int i) {
		//set to integer
		if (mustDelete)
			if (this->value.vt!=VT_EMPTY) {
				VariantClear(&this->value);
			}
		mustDelete=true;
		value.vt=VT_I4;
		value.lVal=i;		
	}

	void Set(bool b) {
		//set to bool
		if (mustDelete)
			if (this->value.vt!=VT_EMPTY) {
				VariantClear(&this->value);
			}
		mustDelete=true;
		value.vt=VT_BOOL;
		value.lVal=(b)?VARIANT_TRUE:VARIANT_FALSE;		
	}

	void Set(VARIANT v) {
		//set to variant, note: this creates a copy and owns the copy
		if (mustDelete)
			if (this->value.vt!=VT_EMPTY) {
				VariantClear(&this->value);
			}
		mustDelete=true;
		value.vt=VT_VARIANT;
		VariantCopy(&value,&v);
	}

};

